//---------------------------------------------------------------------------
#ifndef OVPrinterH
#define OVPrinterH
/**************************************************************
*                                                             *
*   (C) Oleg V. Paschenko, 2000                               *
*                                                             *
*   http://ovp.50megs.com       mailto: ovp@npi.tpu.ru        *
*                                                             *
***************************************************************/
#include <Graphics.hpp>
#include <extctrls.hpp>
#include <Printers.hpp>
#include <vector.h>

namespace OVPrintout
{
//***************************************************************************
class PACKAGE EOVPrintout: public Exception {
public:
   EOVPrintout(const AnsiString Msg) :Exception(Msg) {}
};

//---------------------------------------------------------------------------
class PACKAGE TCustomOVPrintout;
typedef void __fastcall (__closure * TOVPrintoutFlagEvent)(TCustomOVPrintout * Sender, bool & Continue);
typedef void __fastcall (__closure * TOVPrintoutNotifyEvent)(TCustomOVPrintout * Sender);

//***************************************************************************
// TCustomOVPrintout
//***************************************************************************
class TOVPreview;

class PACKAGE TCustomOVPrintout : public TComponent {
   friend TOVPreview;

// constructor / destructor
public:
   __fastcall TCustomOVPrintout(TComponent * AOwner);
   virtual __fastcall ~TCustomOVPrintout();

// Checks
private:
   void __fastcall CheckPainting();

// Pages
private:
   std::vector<TMetafile *> FPages;
   void __fastcall AddPage();
   void __fastcall ClearPages();
   int __fastcall GetPageCount() { return FPages.size(); }
   __property TMetafile * Pages[int PageNo] = { read = GetPages };
   TMetafile * __fastcall GetPages(int pageno) { return FPages[pageno-1]; }
   int FPageNo;
public:
   __property int PageNo = { read = FPageNo };
   __property int PageCount = { read = GetPageCount };

// Page layout
private:
   TPrinter * Ptr;
   TPrinterOrientation FOrientation;
   void __fastcall SetOrientation(TPrinterOrientation Value);
__published:
   __property TPrinterOrientation Orientation =
      { read = FOrientation, write = SetOrientation };
private:
   int FMarginLeftMM, FMarginTopMM, FMarginRightMM, FMarginBottomMM;
   int FMarginLeft, FMarginTop, FMarginRight, FMarginBottom;
   void __fastcall SetMarginLeftMM(int Value);
   void __fastcall SetMarginTopMM(int Value);
   void __fastcall SetMarginRightMM(int Value);
   void __fastcall SetMarginBottomMM(int Value);
   int FPageWidth, FPageHeight;
   void __fastcall TreateMargins();
   void __fastcall SetViewPort();
public:
   // all *MM margins in 0.01 mm units relative to physical page size;
   // 0 = min possible;
   // all painting coordinates relates to left and top margins;
   // right and bottom margins did not clip drawing,
   // but affects PageWidth and PageHeights properties
   void __fastcall SetMarginsMM(int LeftMM, int TopMM, int RightMM, int BottomMM);
__published:
   __property int MarginLeftMM   = { read = FMarginLeftMM, write = SetMarginLeftMM };
   __property int MarginTopMM    = { read = FMarginTopMM, write = SetMarginTopMM };
   __property int MarginRightMM  = { read = FMarginRightMM, write = SetMarginRightMM };
   __property int MarginBottomMM = { read = FMarginBottomMM, write = SetMarginBottomMM };
   // following not-MM margins are in pixels; relative to printable area
   __property int MarginLeft   = { read = FMarginLeft };
   __property int MarginTop    = { read = FMarginTop };
   __property int MarginRight  = { read = FMarginRight };
   __property int MarginBottom = { read = FMarginBottom };
public:
      // following in pixels
   __property int PageWidth    = { read = FPageWidth };
   __property int PageHeight   = { read = FPageHeight };

// Creating documents
//    low-level
private:
   bool FDocStarted;
   bool __fastcall GetPageStarted() { return FCanvas != 0; }
   bool FDone;
   TOVPrintoutFlagEvent FOnBeginDoc;
   TOVPrintoutNotifyEvent FOnFirstPage;
   TOVPrintoutNotifyEvent FOnEndDoc;
   TOVPrintoutNotifyEvent FOnBeginPage;
   TOVPrintoutNotifyEvent FOnEndPage;
protected:
   virtual void __fastcall DoFirstPage();
public:
   __property bool DocStarted = { read = FDocStarted };
   __property bool PageStarted = { read = GetPageStarted };
   __property bool Done = { read = FDone };
   AnsiString Title;
   void __fastcall BeginDoc();
   void __fastcall EndDoc();
   void __fastcall Abort();
   void __fastcall BeginPage();
   void __fastcall EndPage();
   void __fastcall NewPage();
   void __fastcall GotoPage(int pageno);
protected: // to be __published:
   __property TOVPrintoutFlagEvent OnBeginDoc  = { read=FOnBeginDoc, write=FOnBeginDoc };
   __property TOVPrintoutNotifyEvent OnFirstPage  = { read=FOnFirstPage, write=FOnFirstPage };
   __property TOVPrintoutNotifyEvent OnEndDoc  = { read=FOnEndDoc, write=FOnEndDoc };
   __property TOVPrintoutNotifyEvent OnBeginPage  = { read=FOnBeginPage, write=FOnBeginPage };
   __property TOVPrintoutNotifyEvent OnEndPage  = { read=FOnEndPage, write=FOnEndPage };

//    high-level doc maintenance for derived classes
public:
   virtual TModalResult __fastcall CreateDoc();
protected:
   virtual TModalResult __fastcall GetParams() { return mrOk; }
   virtual void __fastcall PrepareData() { }
   virtual void __fastcall PaintDoc();
private:
   TOVPrintoutNotifyEvent FOnPaintDoc;
   TOVPrintoutFlagEvent FOnPaintPage;
protected: // to be __published:
   __property TOVPrintoutNotifyEvent OnPaintDoc  = { read=FOnPaintDoc, write=FOnPaintDoc };
   __property TOVPrintoutFlagEvent OnPaintPage  = { read=FOnPaintPage, write=FOnPaintPage };

// Printer capabilities and margins
private:
   int FPrinterWidth, FPrinterHeight;
   int FDPIX, FDPIY;
   int FPhysPrinterWidth, FPhysPrinterHeight;
   int FPhysMarginX, FPhysMarginY;
   int FMarginX, FMarginY;
public:
   void __fastcall GetPrinterCaps();
   __property int PrinterWidth = { read = FPrinterWidth };           // pixels
   __property int PrinterHeight = { read = FPrinterHeight };
   __property int DPIX = { read = FDPIX };                           // dpi
   __property int DPIY = { read = FDPIY };
   __property int PhysPrinterWidth = { read = FPhysPrinterWidth };   // pixels
   __property int PhysPrinterHeight = { read = FPhysPrinterHeight };
   __property int PhysMarginX = { read = FPhysMarginX };             // pixels
   __property int PhysMarginY = { read = FPhysMarginY };

// Painting
private:
   TMetafileCanvas * FCanvas;
   TCanvas * __fastcall GetCanvas();
   void __fastcall CreateCanvas(bool Old);
   void __fastcall DeleteCanvas();
   TBrush * CurBrush;
   TPen * CurPen;
   TFont * CurFont;
   int CurCopyMode;
   void __fastcall SaveCanvasSettings();
   void __fastcall RestoreCanvasSettings();
public:
   __property TCanvas * Canvas = { read = GetCanvas };

public:
// Printing
   int Copies;
   void __fastcall PrintPages(int FromPage, int ToPage);
   void __fastcall Print(bool ShowDialog);

// Default preview
protected:
   void __fastcall CreateDefaultPreviewForm();
public:
   void __fastcall DefaultPreview();
   void __fastcall DefaultPreviewModal();

// file operations
public:
   static const int SignatureLength = 16;
private:
   bool Loading;
   struct TDataToSave {
      char Signature[SignatureLength];
      char Title[256];
      TPrinterOrientation Orientation;
      int MarginLeftMM, MarginTopMM, MarginRightMM, MarginBottomMM;
      int PageCount;
   };
public:
   void __fastcall SaveToFile(const AnsiString FileName);
   void __fastcall LoadFromFile(const AnsiString FileName);

// some service functions
public:
   int MMToPixelsX(int MM) { return MulDiv(MM,DPIX,2540); }
   int MMToPixelsY(int MM) { return MulDiv(MM,DPIY,2540); }
   int PixelsToMMX(int Pixels) { return MulDiv(Pixels,2540,DPIX); }
   int PixelsToMMY(int Pixels) { return MulDiv(Pixels,2540,DPIY); }
   int __fastcall TextHeight(bool WithExtLeading = false);
   void __fastcall TextRectLeft(const TRect & Rect, AnsiString Text);
   void __fastcall TextRectRight(const TRect & Rect, AnsiString Text);
   void __fastcall TextRectCenter(const TRect & Rect, AnsiString Text);
};
//***************************************************************************
// TOVPrintout
//***************************************************************************
class PACKAGE TOVPrintout: public TCustomOVPrintout {
public:
   __fastcall TOVPrintout(TComponent * AOwner) : TCustomOVPrintout(AOwner) {}
__published:
   __property OnBeginDoc;
   __property OnFirstPage;
   __property OnEndDoc;
   __property OnBeginPage;
   __property OnEndPage;
   __property OnPaintDoc;
   __property OnPaintPage;
};
//***************************************************************************
// TOVPreview
//***************************************************************************
class TOVPreview: public TScrollBox {
public:
   enum TZoomKind { zkWidth, zkHeight, zkPage, zkAbs };
private:
   vector<TPaintBox *> Pages;
   void __fastcall ClearPages();
   void __fastcall CreatePages();
   void __fastcall PaintPage(TObject * Sender);

   TCustomOVPrintout * FOVPrintout;
   void __fastcall SetOVPrintout(TCustomOVPrintout * Value);
   void __fastcall DeleteOVPrintout();
   bool PrintoutIsDestroying;
   void __fastcall Notification(TComponent* AComponent, TOperation Operation);
   bool FTakeOwnership;
   void __fastcall SetTakeOwnership(bool Value);
   void __fastcall ChangeOwnership();
   TNotifyEvent FOnPrintoutChange;

   int FCurrentPage;
   void __fastcall SetFCurrentPage(int Value);
   void __fastcall SetCurrentPage(int Value);
   TNotifyEvent FOnPageChange;
   void __fastcall CheckPageChange();

   bool FTracking;
   void __fastcall SetTracking(bool value);

   TZoomKind FZoomKind;
   void __fastcall SetZoomKind(TZoomKind value);
   int FZoom;
   void __fastcall SetZoom(int value);
   TNotifyEvent FOnZoomChange;

   int FPreviewMargin;
   int FrameHeight;
   void __fastcall AdjustPBSize();
protected:
   DYNAMIC void __fastcall Resize();
   virtual void __fastcall WndProc(TMessage &Messsage);
   void __fastcall DoZoomChange();
public:
   __fastcall TOVPreview(TComponent * Owner);
   __fastcall ~TOVPreview();
   __property int CurrentPage = { read = FCurrentPage, write = SetCurrentPage };
   void __fastcall ScrollUp();
   void __fastcall ScrollDown();
   void __fastcall ScrollToEnd();
   __property bool TakeOwnership = { read = FTakeOwnership, write = SetTakeOwnership, default = false };
__published:
   __property TCustomOVPrintout * OVPrintout = {read = FOVPrintout, write = SetOVPrintout, default = 0 };
   __property bool Tracking  = { read=FTracking, write=SetTracking, default = false };
   __property TZoomKind ZoomKind  = { read=FZoomKind, write=SetZoomKind, default = zkWidth };
   __property int Zoom  = { read=FZoom, write=SetZoom, default = 100 };
   __property int PreviewMargin  = { read=FPreviewMargin, write=FPreviewMargin, default = 16 };
   __property TNotifyEvent OnPrintoutChange  = { read=FOnPrintoutChange, write=FOnPrintoutChange };
   __property TNotifyEvent OnPageChange  = { read=FOnPageChange, write=FOnPageChange, default = 0 };
   __property TNotifyEvent OnZoomChange  = { read=FOnZoomChange, write=FOnZoomChange };
};
//***************************************************************************
}  // OVPrintout
#if !defined(NO_IMPLICIT_NAMESPACE_USE)
using namespace OVPrintout;
#endif

#endif
